require "vizshot_gfdnavi"
require "file_gfdnavi"
require "fileutils"

class AnalysisController < ApplicationController
  layout "gfdnavi"

  before_filter :login_required, :only => :save

  def index
    if session[:analysis]
      @analysis = session[:analysis]
    else
      @analysis = Analysis.new
      clear_diagrams()
      clear_temp_variables()
    end
    session[:analysis] = @analysis
    @variables = all_variables()

    @function_input_nums, @functions = function_set(@analysis)

    if (diagrams = session[:diagrams])
      @diagrams = diagrams.sort.collect{|i,dia| diagram_to_atdiagram(dia)}
    end
  end

  def clear
    session[:analysis] = nil
    clear_diagrams()
    clear_variables()
    redirect_to(:action => "index")
  end

  def variables_selected
    unless request.xhr?
      clear
      return
    end
    session[:analysis] = variables_set(params["variables"])
    action_type_selected
  end

  def action_type_selected
    unless request.xhr?
      clear
      return
    end
    @analysis = session[:analysis] || Analysis.new
    @analysis.action_type = params[:action_type] if params[:action_type]
    if @analysis.action_type == 0 # draw
      if @analysis.dimensions.length >= 2
        @analysis.draw_type = 1 # tone and contour
      else
        @analysis.draw_type = 0 # line
      end
    end
    session[:analysis] = @analysis
    messages = ""
    if @analysis.variables.length == 0
      @analysis.variable_clear
    elsif @analysis.dimensions.length == 0
      messages = "all selected variables must have the same dimensions"
    end

    if Analysis::ACTION_TYPE[@analysis.action_type] == "analysis"
      @function_input_nums, @functions = function_set(@analysis)
    end
    render :update do |page|
      page.replace_html :messages, messages
      page.replace_html :dimensions_setting, render(:partial => "dimension_option")
      case Analysis::ACTION_TYPE[@analysis.action_type]
      when "draw"
        page.replace_html :draw_settings, render(:partial => "draw_settings")
        page.replace_html :popular_diagrams, render(:partial => "popular_diagrams")
      when "analysis"
        page.replace_html :analysis_settings, render(:partial => "analysis_settings")
        page.replace_html :popular_diagrams, ""
      end

      page.replace_html :script, <<"EOF"
<script>
  action_type = #{@analysis.action_type};
  executable = #{@analysis.dimensions.length > 0};
  afterCallBack();
</script>
EOF
    end

  end

  def function_input_num_selected
    unless request.xhr?
      clear
      return
    end
    @analysis = session[:analysis] || Analysis.new
    @analysis.input_num = params[:function_input_num]
    @function_input_nums, @functions = function_set(@analysis)
    render :update do |page|
      page.replace_html :analysis_function_settings, render(:partial => "analysis_function_settings")
    end
  end


  def execute
    action_type = params[:action_type]
    if request.xhr?
      analysis = session[:analysis]
      unless analysis
        render :update do |page|
          page.replace_html :messages, "operation failed. please reload the page."
        end
        return
      end
    else
      analysis = variables_set(params["variables"])
      analysis.draw_share = false
      analysis.action_type = Analysis::ACTION_TYPE.index(action_type)
    end
    analysis.attributes = params[:analysis]
    session[:analysis] = analysis
    case action_type
    when "draw"
      draw(analysis)
      return
    when "analysis"
      if request.xhr?
        analysis(analysis)
        return
      end
    end
    raise "invalid action (#{action_type})"
  end

  def download_diagram_script_and_data
    id = params[:id]
    diagram = get_temp_diagram(id)
    if diagram
      viz = diagram[:vizshot]
      if viz
        dir = work_dir
        NumRu::VizShot.dumpdir = dir
        paths = viz.dump_code_and_data("gfdnavi", true)
#        names = paths.collect{|path| File.basename(path)}.join(" ")
#        command = "cd #{dir} && tar zcvf #{File.basename(work_prefix)}.tar.gz #{names}"
        names = paths.collect{|path| File.basename(path)}
        command = "cd #{dir} && ruby -r #{File.join(Dir.pwd,"lib/tar")} -e 'Tar.zcf(\"#{File.basename(work_prefix)}.tar.gz\", #{names.inspect})'"
        begin
	  res = system_with_error(command, 19)
          if res[0]
            send_file "#{work_prefix}.tar.gz", :filename => "gfdnavi.tar.gz"
            return
	  else
	     message = res[1]
          end
        ensure
          File.delete(*paths)
        end
      else
        message = "cannot find data to download"
      end
    else
      message = "cannot find diagram"
    end
    flash[:messages] = "failed to download<br/>" + message
    redirect_to :action => "index"
  end

  def download_data
    id = params[:id]
    var = get_temp_variable(id)
    if var
      name = var.file_and_variable_names
      if name
        send_file name[0], :filename => "gfdnavi.nc"
        return
      end
    end
    render :update do |page|
      page.replace_html :messages, "cannot download data"
    end
  end

  def save
    if GFDNAVI_DISABLE_USER || !(user = session[:user])
      redirect_to :action => "index"
      return
    end
    if request.get?
      @id = params[:id]
      @type = params[:type]
      @groups = Group.find_for_user(user)
      @user = user
      case @type
      when "diagram"
        @suffix = "png"
      when "data"
        @name = get_temp_variable(@id).name
        @suffix = "nc"
      end
      render
      return
    end
    id = params[:id]
    type = params[:type]
    name = params[:name]
    path = params[:path]
    description = params[:description]
    keywords = params[:keywords]
    group = params[:group]
    case type
    when "diagram"
      var = Variable.new
      var.gphysizable = false
      diagram = get_temp_diagram(id)
      viz = diagram[:vizshot]
      ids = viz.get_variables.collect{|v|
        if Variable===v && ! v.id
          from = v.file_and_variable_names[0]
          to, pa = auto_file_path("data")
          File.move(from, to)
          v.path = pa + Variable::DIR_VAR_DELIMITER + v.name
          v.save!
        else
          v = Variable===v ? v : Variable.find(v)
        end
        var.references_tmp << v
        v.id
      }
      var.keyword_attributes.build(:name => "vizshot", :value => viz.to_yaml)
      path = "/#{path}.png"
      from = diagrams_dir+"/"+diagram[:fname]
    when "data"
      var = get_temp_variable(id)
      var.gphysizable = true
      path = "/#{path}.nc#{Variable::DIR_VAR_DELIMITER}#{name}"
      from = var.file_and_variable_names[0]
    end
    var.name = name
    var.path = "/usr/#{user.login}#{path}"
    var.user = user
    var.group = group = Group.find_by_name(group)
    to = var.file_and_variable_names[0]
    File.makedirs( File.dirname(to) )
    var.keyword_attributes.build(:name => "description", :value => description) unless description == ""
    keywords.each{|k,v|
      if Hash === v && (name=v["name"])!="" && (val=v["value"])!=""
        if val.to_i.to_s == val
          val = NArray[val.to_i]
        elsif val.to_f.to_s == val
          val = NArray[val.to_f]
        end
        var.keyword_attributes.build(:name => name, :value => val)
      end
    }
    if var.save
#      var.references_tmp.each{|v|
#        var.references << v
#      }
      messages = "successfully saved"
      case type
      when "diagram"
        File.copy( from, to )
        diagram[:saved] = true
      when "data"
        File.move( from, to )
        session[:temp_variables_list].delete var
        session[:variables_list].push var
      end
    else
      messages = "failed to save<br/>"
      messages += var.errors.full_messages.join("<br/>") if var.errors
    end
    flash[:messages] = messages
    redirect_to :action => :index
  end

  def delete_diagram
    id = params[:id].to_i
    if (diagram = session[:diagrams][id])
      session[:diagrams].delete(id)
      fname = diagrams_dir + "/" + diagram[:fname]
      File.delete(fname)
    end
    render :nothing => true
  end

  def pile_up
    return unless request.xhr?
    lower_id = params[:lower].to_i
    upper_id = params[:upper].to_i
    if (diagrams = session[:diagrams])
      lower = diagrams[lower_id]
      upper = diagrams[upper_id]
      if lower && upper
        viz = lower[:vizshot].add(upper[:vizshot])
        viz_to_diagram([viz], session[:analysis], true)
        return
      end
    end
    render :nothing => true
  end

  private
  def all_variables
    vars = session[:variables_list] || []
    variables = vars.collect{|v|
      attrs = v.keyword_attributes.collect{|attr| [attr.name, attr.value]}
      selected = @analysis.variables.include?(v)
      [v.id, v.name, attrs, selected, false, v.path]
    }
    temp_vars = session[:temp_variables_list] || []
    for i in 0...temp_vars.length
      v = temp_vars[i]
      attrs = v.keyword_attributes.collect{|attr| [attr.name, attr.value]}
      selected = @analysis.variables.include?(v)
      variables.push [i, v.name, attrs, selected, true, v.path]
    end
    return variables
  end

  def variables_set(params_vars)
    analysis = session[:analysis] || Analysis.new
    analysis.variable_clear
    params_vars && params_vars.each do |k,v|
      if /temp_(\d*)/ =~ k
        var = session[:temp_variables_list][$1.to_i]
      else
        if k.to_i.to_s == k
          var = Variable.find_for_user(session[:user], k.to_i)
        else
          var = Variable.find_for_user(session[:user], :first, :conditions => ["path = ?", k])
        end
        if var
          session[:variables_list] ||= Array.new
          session[:variables_list].push(var) unless session[:variables_list].include?(var)
        end
      end
      if var && v=="1"
        analysis.variables.push var unless analysis.variables.include?(var)
      else
        analysis.variables.delete var
      end
    end
    return analysis
  end

  def function_set(analysis)
    input_nums = Array.new
    nvars = analysis.variables.length
    user = session[:user]
    nvars.times{|ii|
      i = ii+1
      if (nvars.to_f/i).to_i*i == nvars && Function.find_for_user(user, :first, :conditions => ["input_num=?", i])
        input_nums.push(i)
      end
    }
    if input_nums.length > 0
      analysis.input_num = input_nums[-1] unless analysis.input_num
      functions = Function.find_for_user(user, :all, :conditions => ["input_num = ?", analysis.input_num], :order => [:name])
    else
      functions = Array.new
    end
    return [input_nums, functions]
  end

#draw
  def draw(analysis)
    NumRu::VizShot.dumpdir = work_dir
    res = analysis.get_vizshots(work_prefix)
    unless res[0]
      render :update do |page|
        page.replace_html :messages, res[1]
      end
      return
    end
    viz_to_diagram(res[1], analysis)
  end

  def viz_to_diagram(vizs, analysis=nil, keep=nil)
    keep = keep.nil? ? analysis.draw_keep : keep
    diagrams = Array.new
    draw_or_cache = [0,0]
    vizs.length.times{|i|
      viz = vizs[i]
      msss = nil
      diagram_cache = nil
      if analysis && (diagram_cache = find_diagram_cache(analysis, viz))
        draw_or_cache[1] += 1
        flag = true
      else
        viz.dump_code( nil, true, :image_dump => true )
        wdir = work_dir
        wdir.gsub!(/\//,File::ALT_SEPARATOR) if File::ALT_SEPARATOR
        command = "cd #{wdir} && ruby \"#{work_prefix}.rb\""
        flag, mess = system_with_error(command, 10)
        ActiveRecord::Base.connection.reconnect!
        if flag && analysis
          push_diagram_cache(analysis, viz)
        end
        draw_or_cache[0] += 1
      end
      if flag
        diagram = move_diagram(i==0 ? keep : true, diagram_cache)
      end
      unless diagram
        messages = "could not draw diagram<br/>"
	if mess
	  messages += mess
	else
	  messages += "[BUG] cannot move diagram file"
	end
        if request.xhr?
          render :update do |page|
            page.replace_html :messages, messages
          end
        else
          render :text => messages
        end
        return
      end
      diagram[:vizshot] = viz
      id = diagram[:id]
      session[:diagrams] ||= Hash.new
      session[:diagrams][id] = diagram
      diagrams.push diagram
    }
    show_diagram(diagrams, keep, draw_or_cache)
  end

  def show_diagram(diagrams, keep, draw_or_cache)
    atdiagrams = diagrams.collect{|diagram| diagram_to_atdiagram(diagram)}
    if request.xhr?
      @analysis = session[:analysis]
      render(:update){|page|
        page.replace_html :messages, ""
        if GFDNAVI_BENCHMARK
          ids = atdiagrams.collect{|di| di[2]}
          html = host_information_table
          html +=<<-"EOS"
            Total number of diagrams is #{ids.length}<br/>
            Cached diagram: #{draw_or_cache[1]}<br/>
            Drawed diagram: #{draw_or_cache[0]}
          EOS
          page << "benchMark.set(['#{ids.join('\',\'')}'], '#{escape_javascript(html)}');"
          @benchmark = true
        end
        atdiagrams.length.times{|i|
          @diagram = atdiagrams[i]
          if keep || i>0
            page.insert_html :bottom, :diagrams, render(:partial => "diagram")
          else
            page.replace_html :diagrams, render(:partial => "diagram")
          end
        }
      }
      return
    else
      redirect_to :action => "index"
    end
  end

  def diagram_to_atdiagram(diagram)
    fname = diagram[:fname]
    diagram_id = diagram[:id]
    path = "/diagrams/#{File.basename(diagrams_dir)}/#{fname}"
    id = "diagram_#{diagram_id}"
    return [id, path, diagram_id, diagram[:saved], diagram[:vizshot]]
  end

  def diagrams_dir
    if "ActionController::TestSession" == session.class.to_s   # for test
      dir = "#{GFDNAVI_DIAGRAM_PATH}/test"
      FileUtils.makedirs(dir) unless File.exist?(dir)
      return dir
    end
    dir = session[:diagrams_dir]
    unless dir
      dir = File.temp_name(GFDNAVI_DIAGRAM_PATH)
      FileUtils.makedirs(dir)
      session[:diagrams_dir] = dir
    end
    return dir
  end

  def work_prefix(flag = true)
    work_dir(flag) + "/temp"
  end

  def work_dir(flag = true)
    dir = "/#{session.session_id}"
    dir = "/test" if dir == "/"
    full_dir = GFDNAVI_WORK_PATH + dir
    FileUtils.makedirs(full_dir) unless File.exist?(full_dir)
    return flag ? full_dir : dir
  end

  def move_diagram(flag=false, diagram_cache=nil)
    dir = diagrams_dir
    return nil unless dir
    page = "000"
    if flag
      files = Dir[dir+"/gfdnavi_*.png"].sort
      if files.length >0
        files[-1] =~ /.*gfdnavi_(\d*).png$/
        page = $1.next
      end
    else
      clear_diagrams
    end
    fname = "gfdnavi_#{page}.png"
    if diagram_cache
      File.copy(diagram_cache, "#{dir}/#{fname}")
    else
      File.rename(work_prefix+"_001.png", "#{dir}/#{fname}")
    end
    return {:fname => fname, :id => page.to_i}
  end

  def find_diagram_cache(analysis, viz)
    return nil if GFDNAVI_DISABLE_DIAGRAM_CACHE
    analysis.variables.each{|v|
      unless v.id
        return nil
      end
      v.diagram_caches(true).each{|dc|
        if dc.uri_params == Analysis.uri_params_from_vizshot(viz)
          if (dcs = dc.diagram_cache_sessions.find_by_session(session.session_id))
            if analysis.draw_share && !dcs.share
              dcs.share = true
              dcs.save!
            end
          else
            dcs = DiagramCacheSession.new
            dcs.diagram_cache = dc
            dcs.session = session.session_id
            dcs.save!
          end
          return dc.path
        end
      }
    }
    return nil
  end

  def push_diagram_cache(analysis, viz)
    return nil if GFDNAVI_DISABLE_DIAGRAM_CACHE
    uri_params = Analysis.uri_params_from_vizshot(viz)
    return nil if uri_params.nil?
    dc = DiagramCache.new
    dc.uri_params = uri_params
    fname = File.temp_name(GFDNAVI_DIAGRAM_CACHE_PATH, ".png")
    File.copy(work_prefix+"_001.png", fname)
    dc.path = fname
    dc.label = analysis.diagram_label
    dc.save!
    viz.get_variables.each{|v|
      if Variable===v && ! v.id
        return nil
      end
      v = Variable===v ? v : Variable.find(v)
      dcd = DiagramCacheDatum.new
      dcd.diagram_cache = dc
      dcd.variable = v
      dcd.save!
      dcs = DiagramCacheSession.new
      dcs.diagram_cache = dc
      dcs.session = session.session_id
      dcs.share = analysis.draw_share
      dcs.save!
    }
  end

  def clear_diagrams
    dir = diagrams_dir
    return nil unless dir
    files = Dir[dir+"/gfdnavi_*.png"].each do |file|
      File.delete(file)
    end
    session[:diagrams] = nil
  end

# analysis
  def analysis(analysis)
    @analysis = analysis
    function = analysis.function
    vlen = analysis.variables.length
    input_num = analysis.input_num
    unless function && input_num == function.input_num && (input_num==1 || analysis.function_variables_order)
      debug( analysis.inspect )
      messages = "invalid operation was required"
    else
      messages = ""
      new_vars = 0
      (vlen/input_num).times{|n|
        if input_num == 1
          variables = [ analysis.variables[n] ]
        else
          variables = analysis.function_variables_order[n].collect{|i|
            analysis.variables[i]
          }
        end
        vname = variables.collect{|v|
          v.name
        }.join("_")
        outputs = function.function_outputs
        ovnames = Array.new
        attrs = Array.new
        outputs.length.times{|i|
          output = outputs[i]
          ovnames[i] = "#{vname}_#{output.subscript}"
          attrs[i] = [["analysis", output.description + " calculated from " + vname]]
        }
        vars, mes = create_new_variable(variables, analysis.index, ovnames, analysis.function_arguments, function.script, function.user_id, attrs)
        if vars
          session[:temp_variables_list] = Array.new unless session[:temp_variables_list]
          vars.each{|var| session[:temp_variables_list].push var}
          new_vars += vars.length
        else
          messages = "faild to create some variable<br/>" + mes
        end
      }
      @variables = all_variables()
      render :update do |page|
        page.replace_html :messages, messages
        page.replace_html :variables_body, render(:partial => "variables")
        if GFDNAVI_BENCHMARK
          html = host_information_table
          html +=<<-"EOS"
            Number of created variable is #{new_vars}.<br/>
          EOS
          page << "benchMark.set(null, '#{escape_javascript(html)}');"
          page << "benchMark.complete();"
        end
      end
      return
    end
    render :update do |page|
      page.replace_html :messages, messages
    end
  end

  def create_new_variable(variables, index, ovnames, func_args, script, uid, attrs=nil)
    code = <<"EOF"
require "numru/gphys"
include NumRu

gphys = Array.new
EOF
    nvars = variables.length
    names = variables.collect{|var| var.file_and_variable_names}
    nvars.times do |i|
      code +=<<"EOF"
arg0 = ARGV.shift
if arg0
  arg0 = arg0.split(",")
  fname#{i} = arg0.length==1 ? arg0.first : arg0
else
  fname#{i} = "gfdnavi#{i}.nc"
end
vname#{i} = "#{names[i][1]}"
gphys[#{i}] = GPhys::IO.open(fname#{i}, vname#{i})
gphys[#{i}] = gphys[#{i}].cut(#{index.join(", ")})
EOF
    end
    code +=<<"EOF"
ovnames = Array.new
EOF
    ovnames.length.times do |i|
      code +=<<"EOF"
ovnames[#{i}] = "#{ovnames[i]}"
EOF
    end
    if script
      code +=<<"EOF"
args = Array.new
EOF
      arg = Array.new
      if Array===func_args
        func_args.length.times do |i|
          code +=<<"EOF"
args[#{i}] = #{func_args[i]}
EOF
          arg.push "arg#{i}"
        end
      end
      nvars.times{|i| arg.push "gphys#{i}"}
      arg = arg.join(", ")
      code +=<<"EOF"
proc = Proc.new{|#{arg}|
EOF
      if uid
        code +=<<"EOF"
  $SAFE = 3
EOF
      end
      code +=<<"EOF"
  #{script}
}
gphys = proc.call(*(args+gphys))
EOF
    end
    code +=<<"EOF"
ofname = ARGV.shift || "output.nc"
file = NetCDF.create(ofname, true)
gphys.length.times do |i|
  gp = gphys[i]
  gp.name = ovnames[i]
  GPhys::IO.write( file, gp )
end
file.close
EOF

    args = names.collect{|f,v| Array===f ? f.join(",") : f}
    prefix = work_prefix
    files = Dir["#{prefix}_*.nc"].sort
    if files.length >0
      files[-1] =~ /.*_(\d*).nc$/
      num = $1.next
    else
      num = "000"
    end
    ofname = "#{work_prefix(false)}_#{num}.nc"
    args.push work_dir + "/" + File.basename(ofname)
    args = args.join(" ")
    debug( code )
    script_name = work_prefix+".rb"
    File.open(script_name,"w"){|file|
      file.print code
    }
    command = "ruby #{script_name} #{args}"
    res = system_with_error(command, 19)
    if res[0]
      nvars = ovnames.length
      vars = Array.new(nvars)
      nvars.times{|i|
        name = ovnames[i]
        var = Variable.new({:path => "temporary:#{ofname}#{Variable::DIR_VAR_DELIMITER}#{name}",
                            :name => name })
        variables.each{|v|
          var.references_tmp << v
        }
        if variables[0].starttime
#          var.starttime = Date.new()
#          var.endtime = Date.new()
        end
        if attrs && attrs[i]
          attrs[i].each do |k,v|
            var.keyword_attributes.build({:name => k, :value => v})
          end
        end
        vars[i] = var
      }
      return vars, nil
    else
      return nil, res[1]
    end
  end

#common
  def get_temp_variable(id)
    variables = session[:temp_variables_list]
    if variables
      return variables[id.to_i]
    else
      return nil
    end
  end

  def get_temp_diagram(id)
    diagrams = session[:diagrams]
    if diagrams
      return diagrams[id.to_i]
    else
      return nil
    end
  end

  def clear_temp_variables
    files = Dir["#{work_prefix}_*.nc"].each do |file|
      File.delete(file)
    end
    session[:temp_variables_list] = nil
  end

  def clear_variables
    clear_temp_variables()
    session[:variables_list] = nil
  end

  def auto_file_path(type)
    case type
    when "data"
      suffix = "nc"
    when "diagram"
      suffix = "png"
    end
    path = user_path+"/auto"
    FileUtils.makedirs(path) unless File.exist?(path)
    last = Dir["#{path}/gfdnavi_\d+\.#{suffix}"][0]
    if last
      last =~ /gfdnavi_(\d+)\./
      num = $1.next
    else
      num = "0000"
    end
    fname = "gfdnavi_#{num}.#{suffix}"
    full_path = "#{path}/#{fname}"
    path = "/usr/#{session[:user].login}/auto/#{fname}"
    return [full_path, path]
  end


  def system_with_error(command, prio)
    debug( command )
    if /linux/ =~ Config::CONFIG["arch"]
      rr,rw = IO.pipe
      er,ew = IO.pipe
      pid = fork {
        rr.close
        er.close
        STDERR.reopen(ew)
        begin
          Process.setpriority(Process::PRIO_PROCESS, 0, prio)
        rescue Errno::EACCES
        end
        res = system(command)
        rw.print res
        rw.close
        ew.close
      }
      rw.close
      ew.close
      Process.waitpid(pid)
      res = rr.gets == "true" ? true : false
      mes = er.readlines
      debug( mes.join("") ) unless res
      mes = mes.delete_if{|me| /^\s*from / =~ me }.collect{|me|
        me.sub(/[\/\w\.]*\.rb:\d*:in /,"")
      }
      mes = mes.join("").gsub(/\n/,"<br/>")
      rr.close
      er.close
      if res
        mes = nil
      end
    else
      res = system( command )
      mes = res ? nil : "failed to execute"
    end
    return [res, mes]
  end

end
